unit uDaten;

interface

uses
  windows;
const
  c_xwide=5;
  c_ywide=5;
  c_xywide=c_xwide*c_ywide;

type
  r_koordinaten = record
    x,y: integer;
  end;

  r_single = record
    next: pointer;
    x,y,fluss:integer;
    used: boolean;
  end;

  r_feld= array[1..c_xwide,1..c_ywide] of r_single;

var
  v_data: r_feld;
  v_ablaufx, v_ablaufy: integer;

  v_sqrttab: array[1..c_xywide] of real;

function calcfluss:integer;
function calcdisp:real;

procedure connect(vp_srcpos,vp_targetpos:tpoint);
function koordinaten(vp_x,vp_y:integer):tpoint;
function getorient(vp_pos:tpoint):integer; // 0: Spitze nach oben

function allconnected: boolean; // berprft ob alle Wege zum Ablauf fhren
function follow(vp_pos:tpoint):integer; // Verfolgt den Wasserflu von diesem Weg aus.

implementation

uses
  classes;

function calcfluss:integer;
var
  vp_run, vp_summe: integer;
  pp_startpos,pp_pos:^r_single;
begin
  // Reset
  pp_pos:=@v_data;
  for vp_run:= 1 to (c_xwide)*(c_Ywide)
  do
    begin
      pp_pos^.fluss:=1;
      inc(pp_pos);
    end;

  // Alle Wasserwege verfolgen
  pp_startpos:=@v_data;
  for vp_run:= 1 to (c_xwide)*(c_Ywide)
  do
    begin
      if not (pp_startpos^.next=pp_startpos)
      then
        begin
          pp_pos:=pp_startpos;
          repeat
            pp_pos:=pp_pos^.next;
            inc(pp_pos^.fluss);
          until pp_pos^.next=pp_pos;
        end;
      inc(pp_startpos);
    end;

  // Summe bilden
  vp_summe:=0;
  pp_pos:=@v_data;
  for vp_run:= 1 to (c_xwide)*(c_Ywide)
  do
    begin
      vp_summe:=vp_summe+pp_pos^.fluss;
      inc(pp_pos);
    end;
  result:=vp_summe;
end;

function calcdisp:real;
var
  vp_run: integer;
  vp_disp:real;
  pp_startpos,pp_pos:^r_single;
begin
  // Reset
  pp_pos:=@v_data;
  for vp_run:= 1 to (c_xwide)*(c_Ywide)
  do
    begin
      pp_pos^.fluss:=1;
      inc(pp_pos);
    end;

  // Alle Wasserwege verfolgen
  pp_startpos:=@v_data;
  for vp_run:= 1 to (c_xwide)*(c_Ywide)
  do
    begin
      pp_pos:=pp_startpos;
      while not (pp_pos^.next=pp_pos)
      do
        begin
          pp_pos:=pp_pos^.next;
          inc(pp_pos^.fluss);
        end;
      inc(pp_startpos);
    end;

  // Summe bilden
  vp_disp:=0;
  pp_pos:=@v_data;
  for vp_run:= 1 to (c_xwide)*(c_Ywide)
  do
    begin
      vp_disp:=vp_disp+sqrt(pp_pos^.fluss);
      inc(pp_pos);
    end;
  vp_disp:=(vp_disp-sqrt(c_xwide*c_ywide));
  calcdisp:=vp_disp;
end;

procedure connect(vp_srcpos,vp_targetpos:tpoint);
var
  p_single: ^r_single;
begin
  p_single:=@v_data[vp_srcpos.x,vp_srcpos.y];
  if (vp_targetpos.x=-1)and(vp_targetpos.y=-1)
  then
    p_single^.next:=nil
  else
    p_single^.next:=@v_data[vp_targetpos.x,vp_targetpos.y];
  p_single^.x:=vp_targetpos.x;
  p_single^.y:=vp_targetpos.y;
  p_single^.used:=true;
end;

function koordinaten(vp_x,vp_y:integer):tpoint;
var
  vp_result:tpoint;
begin
  vp_result.x:=vp_x;
  vp_result.y:=vp_y;
  result:=vp_result;
end;

function getorient(vp_pos:tpoint):integer;
begin
  result:=(vp_pos.x + vp_pos.y) mod 2;
end;

function allconnected: boolean; // berprft ob alle Wege zum Ablauf fhren
var
  vp_connected:array[1..c_xwide,1..c_ywide] of byte;
  vp_xrun,vp_yrun,vp_xpos,vp_ypos, vp_xoldpos: integer;
  vp_nokreis:boolean;
begin
  for vp_yrun := 1 to c_ywide do
    for vp_xrun := 1 to c_xwide do
      vp_connected[vp_xrun,vp_yrun]:=0;
  vp_nokreis:=true;
  vp_connected[v_ablaufx,v_ablaufy]:=1;
  for vp_yrun:=1 to c_ywide do
    for vp_xrun:=1 to c_xwide do
      if (vp_nokreis)and((vp_xrun<>v_ablaufx)or(vp_yrun<>v_ablaufy))
      then
        begin
          vp_xpos:=vp_xrun;
          vp_ypos:=vp_yrun;
          repeat
            vp_connected[vp_xpos,vp_ypos]:=vp_connected[vp_xpos,vp_ypos]+2;
            vp_xoldpos:=vp_xpos;
            vp_xpos:=v_data[vp_xpos,vp_ypos].x;
            vp_ypos:=v_data[vp_xoldpos,vp_ypos].y;
          until (vp_connected[vp_xpos,vp_ypos]=1)or
                (vp_connected[vp_xpos,vp_ypos]=2);
          if (vp_connected[vp_xpos,vp_ypos]=2)
          then vp_nokreis:=false // Wasser fliet im Kreis
          else // Wasser fliet ab
            begin
              vp_xpos:=vp_xrun;
              vp_ypos:=vp_yrun;
              repeat
                vp_connected[vp_xpos,vp_ypos]:=1;
                vp_xoldpos:=vp_xpos;
                vp_xpos:=v_data[vp_xpos,vp_ypos].x;
                vp_ypos:=v_data[vp_xoldpos,vp_ypos].y;
              until (vp_connected[vp_xpos,vp_ypos]=1);
            end;
        end;
  result:=vp_nokreis;
end;

{***********************************************************************}
{** Ergebnisse:                                                       **}
{** wenn das Wasser zum Ablauf fliet:       0                        **}
{** wenn der Weg des Wassers irgendwo endet: 1                        **}
{** wenn das Wasser im Kreis fliet:         2                        **}
{***********************************************************************}
function follow(vp_pos:tpoint):integer; // Verfolgt den Wasserflu von diesem Weg aus.
var
  pp_pos,pp_startpos:^r_single;
begin
  pp_pos:=@v_data[vp_pos.x,vp_pos.y];
  pp_startpos:=pp_pos;
  repeat
    pp_pos:=pp_pos^.next;
  until (pp_pos^.next=pp_pos) // Wasser hat den Ablauf erreicht
      or(pp_pos^.next=pp_startpos)   // Wasser fliet im Kreis
      or(pp_pos^.next=nil);              // Wasser hat Ende des Weges erreicht

  if (pp_pos^.next=pp_pos)
  then
    result:=0;
  if (pp_pos^.next=pp_startpos)
  then
    result:=2;
  if (pp_pos^.next=nil)
  then
    result:=1;
end;

end.
